import boto3
import time
import os
from datetime import datetime


def s3_folder_exists(bucket, event_date, event_hour, s3_obj):
    prefix = s3_obj + '/event_date=' + event_date + '/event_hour=' + event_hour
    print(f"Check data availability for {bucket}, {prefix}, event_date = {event_date}, event_hour = {event_hour}")
    session = boto3.session.Session()
    client = session.client('s3')

    response = client.list_objects_v2(
        Bucket=bucket,
        Prefix=prefix,
    )
    return len(response['Contents']) > 0


def add_partition(aws_region, database, table_name, s3_bucket, athena_result_bucket, event_date, event_hour, s3_obj):
    print(f"add_partition {table_name}, {aws_region} begin")
    session = boto3.session.Session(region_name=aws_region)
    client = session.client('athena')
    sql = "ALTER TABLE " + table_name + " ADD PARTITION (event_date= '" \
          + event_date + "', event_hour= '" + event_hour + "') LOCATION " \
          + "'s3://" + s3_bucket + "/" + s3_obj + "/event_date=" \
          + event_date + "/event_hour=" + event_hour + "/'"

    client.start_query_execution(
        QueryString=sql,
        QueryExecutionContext={
            'Database': database
        },
        ResultConfiguration={
            'OutputLocation': 's3://' + athena_result_bucket
        },
        WorkGroup=os.environ['ATHENA_WORKGROUP']
    )
    print(sql)


def main():
    print(f"Starting at {datetime.now()}")
    env = os.environ['ENV']
    aws_region = os.environ['AWS_REGION']
    #table_dict = {'$TABLE_NAME': ['$S3_PREFIX','$DATABASE_NAME','$FLINK_S3_BUCKET']}
    table_dict = {'flink_sensor_telemetry': ['telemetry/event',os.environ['DATABASE_NAME'],os.environ['FLINK_RESULT_BUCKET']],
                  'flink_sensor_telemetry_audit': ['audit-telemetry/event',os.environ['DATABASE_NAME'],os.environ['FLINK_RESULT_BUCKET']],
                  'flink_octarine': ['octarine/event',os.environ['DATABASE_NAME_OCTARINE'],os.environ['FLINK_OCTARINE_RESULT_BUCKET']],
                  'flink_cwpappliance_telemetry': ['click-events',os.environ['DATABASE_NAME_CWP_APPLIANCE_TELEMETRY'],os.environ['FLINK_CWP_APPLIANCE_TELEMETRY_RESULT_BUCKET']],
                  'flink_pds_report': ['pds',os.environ['DATABASE_NAME_APPC_PDS_REPORTS'],os.environ['FLINK_APPC_PDS_REPORTS_RESULT_BUCKET']],
                  'flink_policy_service_update': ['policy-update',os.environ['DATABASE_NAME_POLICY_SERVICE'],os.environ['FLINK_POLICY_SERVICE_RESULT_BUCKET']]}

    athena_result_bucket = f"{os.environ['ATHENA_OUTPUT_BUCKET']}/results"

    now = time.gmtime()
    event_date = time.strftime("%Y-%m-%d", now)
    event_hour = time.strftime("%H", now)

    for key in table_dict:
        try:
            if s3_folder_exists(table_dict[key][2], event_date, event_hour, table_dict[key][0]):
                add_partition(aws_region, table_dict[key][1], key, table_dict[key][2], athena_result_bucket, event_date, event_hour,
                              table_dict[key][0])
        except Exception as error:
            print("An exception occurred:", error)
    print(f"End at {datetime.now()}")

main()